package ru.javakira.simbirshare.account;

import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.web.server.ResponseStatusException;
import ru.javakira.simbirshare.directory.CreateDirectoryRequest;
import ru.javakira.simbirshare.directory.Directory;
import ru.javakira.simbirshare.directory.DirectoryRepository;
import ru.javakira.simbirshare.directory.DirectoryService;
import ru.javakira.simbirshare.jwt.JwtService;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Service
@RequiredArgsConstructor
public class AccountServiceImpl implements AccountService {
    private static final Pattern emailPattern = Pattern.compile("^[\\w-\\.]+@([\\w-]+\\.)+[\\w-]{2,4}$");

    private final AccountProperties properties;
    private final AccountRepository repository;
    private final JwtService jwtService;
    private final PasswordEncoder passwordEncoder;
    private final AuthenticationManager authManager;

    @Override
    public Account account(long id) {
        return repository
                .findById(id)
                .filter(account -> account.getState() != AccountState.deleted)
                .orElseThrow(() -> new ResponseStatusException(
                    HttpStatus.NOT_FOUND,
                    "Account with id %d doesnt exist".formatted(id)
                )
        );
    }

    @Override
    public AccountDto accountDto(long id) {
        return AccountDto.from(account(id));
    }

    @Override
    public AuthResponse signUp(RegisterRequest request) {
        if (!validateEmail(request.getEmail())) {
            throw new ResponseStatusException(
                    HttpStatus.BAD_REQUEST,
                    "Provided invalid email: %s".formatted(request.getEmail())
            );
        }

        if (repository.findByEmail(request.getEmail()).isPresent())
            throw new ResponseStatusException(
                    HttpStatus.BAD_REQUEST,
                    "email %s is busy".formatted(request.getEmail())
            );

        Account account = Account
                .builder()
                .email(request.getEmail())
                .password(passwordEncoder.encode(request.getPassword()))
                .diskSize(properties.getDefaultDiskSize())
                .build();

        repository.save(account);
        return new AuthResponse(account.getId(), jwtService.generateToken(account));
    }

    private boolean validateEmail(String email) {
        Matcher matcher = emailPattern.matcher(email);
        return matcher.matches();
    }

    @Override
    public AuthResponse signIn(AuthRequest request) {
        authManager.authenticate(
                new UsernamePasswordAuthenticationToken(
                        request.getEmail(),
                        request.getPassword()
                )
        );

        Account account = repository.findByEmail(request.getEmail())
                .orElseThrow();

        return new AuthResponse(account.getId(), jwtService.generateToken(account));
    }

    @Override
    public void update(long userId, UpdateRequest updateRequest) {
        Account account = account(userId);
        account.setPassword(passwordEncoder.encode(updateRequest.getPassword()));
        repository.save(account);
    }

    @Override
    public void logout(String jwt) {
        jwtService.logout(jwt);
    }
}
