package ru.javakira.simbirshare.account.admin;

import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.web.server.ResponseStatusException;
import ru.javakira.simbirshare.account.*;
import ru.javakira.simbirshare.directory.Directory;
import ru.javakira.simbirshare.directory.DirectoryRepository;

import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Service
@RequiredArgsConstructor
public class AdminAccountServiceImpl implements AdminAccountService {
    private static final Pattern emailPattern = Pattern.compile("^[\\w-\\.]+@([\\w-]+\\.)+[\\w-]{2,4}$");

    private final AccountRepository repository;
    private final AccountService service;
    private final PasswordEncoder passwordEncoder;
    private final DirectoryRepository directoryRepository;

    @Override
    public AccountDto create(AccountCreateRequest request) {
        //todo можно убрать повторяющийся код
        if (!validateEmail(request.getEmail())) {
            throw new ResponseStatusException(
                    HttpStatus.BAD_REQUEST,
                    "Provided invalid email: %s".formatted(request.getEmail())
            );
        }

        if (repository.findByEmail(request.getEmail()).isPresent())
            throw new ResponseStatusException(
                    HttpStatus.BAD_REQUEST,
                    "email %s is busy".formatted(request.getEmail())
            );

        Account account = Account
                .builder()
                .diskSize(request.getDiskSize())
                .password(passwordEncoder.encode(request.getPassword()))
                .email(request.getEmail())
                .role(request.isAdmin() ? AccountRole.admin : AccountRole.user)
                .build();


        repository.save(account);
        Directory directory = Directory.builder().ownerId(account.getId()).name("user" + account.getId()).build();
        account.setRoot(directory);
        directoryRepository.save(directory);
        repository.save(account);

        return AccountDto.from(account);
    }

    @Override
    public AccountDto put(AccountPutRequest request, long userId) {
        Account account = service.account(userId);

        //todo тут тоже
        if (!validateEmail(request.getEmail())) {
            throw new ResponseStatusException(
                    HttpStatus.BAD_REQUEST,
                    "Provided invalid email: %s".formatted(request.getEmail())
            );
        }

        if (repository.findByEmail(request.getEmail()).filter(account1 -> account.getId() != userId).isPresent())
            throw new ResponseStatusException(
                    HttpStatus.BAD_REQUEST,
                    "email %s is busy".formatted(request.getEmail())
            );

        account.setPassword(passwordEncoder.encode(request.getPassword()));
        account.setRole(request.isAdmin() ? AccountRole.admin : AccountRole.user);
        account.setEmail(request.getEmail());
        account.setDiskSize(request.getDiskSize());

        repository.save(account);
        return AccountDto.from(account);
    }

    @Override
    public void delete(long userId) {
        Account account = service.account(userId);
        account.setState(AccountState.deleted);
        repository.save(account);
    }

    @Override
    public List<AccountDto> accounts(AccountsRequest request) {
        return repository
                .findAll()
                .stream()
                .map(AccountDto::from)
                .toList();
    }

    private boolean validateEmail(String email) {
        Matcher matcher = emailPattern.matcher(email);
        return matcher.matches();
    }
}
