package ru.javakira.simbirshare.directory;

import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.server.ResponseStatusException;
import ru.javakira.simbirshare.account.Account;
import ru.javakira.simbirshare.account.AccountService;
import ru.javakira.simbirshare.file.FileService;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

@Service
@RequiredArgsConstructor
public class DirectoryServiceImpl implements DirectoryService {
    private final DirectoryRepository repository;
    private final AccountService accountService;
    private final FileService fileService;

    @Override
    public Directory directory(long id) {
        return repository
                .findById(id)
                .orElseThrow(() -> new ResponseStatusException(
                    HttpStatus.NOT_FOUND,
                    "directory with id %d doesn't exist".formatted(id)
                )
        );
    }

    @Override
    public DirectoryDto getRoot(long userId) {
        Account account = accountService.account(userId);

        return DirectoryDto.from(
                repository
                        .findAll()
                        .stream()
                        .filter(directory -> directory.getOwnerId() == userId)
                        .filter(directory -> directory.getParentId().isEmpty())
                        .findFirst()
                        .orElseThrow()
        );
    }

    @Override
    public List<Directory> getDirectories(Directory directory) {
        List<Directory> directories = new LinkedList<>();
        directories.add(directory);

        if (directory.getParentId().isPresent())
            directories.addAll(getDirectories(directory(directory.getParentId().orElseThrow())));

        return directories;
    }

    @Override
    public DirectoryDto directoryDto(long userId, long id) {
        Account account = accountService.account(userId);
        Directory directory = directory(id);
        if (directory.getOwnerId() != userId)
            throw new ResponseStatusException(
                    HttpStatus.FORBIDDEN,
                    "Access to directory denied"
            );

        return DirectoryDto.from(directory);
    }

    @Override
    public List<DirectoryDto> getByDirectory(long userId, long id) {
        Account account = accountService.account(userId);
        Directory directory = directory(id);
        if (directory.getOwnerId() != userId)
            throw new ResponseStatusException(
                    HttpStatus.FORBIDDEN,
                    "Access to directory denied"
            );

        return getByDirectory(id);
    }

    @Override
    public DirectoryDto createDirectory(long userId, CreateDirectoryRequest request) {
        Directory parent = directory(request.getParentId());
        Account account = accountService.account(userId);
        if (parent.getOwnerId() != userId)
            throw new ResponseStatusException(
                    HttpStatus.FORBIDDEN,
                    "Access to directory denied"
            );

        Directory directory = Directory
                .builder()
                .parentId(request.getParentId())
                .ownerId(userId)
                .name(request.getName())
                .build();
        repository.save(directory);
        return DirectoryDto.from(directory);
    }

    @Override
    public DirectoryDto putDirectory(long id, PutDirectoryRequest request, long userId) {
        Directory directory = directory(id);

        if (directory.getParentId().isEmpty())
            throw new ResponseStatusException(
                    HttpStatus.BAD_REQUEST,
                    "Cant update root directory"
            );

        if (getChildren(directory).contains(directory) || directory.getId() == request.getParentId())
            throw new ResponseStatusException(
                    HttpStatus.BAD_REQUEST,
                    "Children of directory or directory itself %d cant be used as parent".formatted(id)
            );

        if (directory(request.getParentId()).getOwnerId() != userId)
            throw new ResponseStatusException(
                    HttpStatus.FORBIDDEN,
                    "Access to directory denied"
            );

        directory.setName(request.getName());
        directory.setParentId(request.getParentId());

        repository.save(directory);
        return DirectoryDto.from(directory);
    }

    private List<Directory> getChildren(Directory directory) {
        List<Directory> directories = new ArrayList<>(repository
                .findAll()
                .stream()
                .filter(directory1 -> directory.getParentId().isPresent())
                .filter(directory1 -> directory.getParentId().get() == directory.getId())
                .toList());

        for (Directory directory1 : directories) {
            directories.addAll(getChildren(directory1));
        }

        return directories;
    }

    @Override
    public void delete(long id, long userId) {
        Account account = accountService.account(userId);
        Directory directory = directory(id);
        if (directory.getParentId().isEmpty())
            throw new ResponseStatusException(
                    HttpStatus.BAD_REQUEST,
                    "Cant remove root directory"
            );

        if (directory.getOwnerId() != userId)
            throw new ResponseStatusException(
                    HttpStatus.FORBIDDEN,
                    "Access to directory denied"
            );

        repository.delete(directory);
    }

    @Override
    public void delete(long id) {
        Directory directory = directory(id);
        repository.delete(directory);
    }

    @Override
    public DirectoryDto directoryDto(long id) {
        return DirectoryDto.from(directory(id));
    }

    @Override
    public List<DirectoryDto> getByDirectory(long dirId) {
        return repository
                .findAll()
                .stream()
                .filter(directory -> directory.getParentId().isPresent())
                .filter(directory1 -> directory1.getParentId().get() == dirId)
                .map(DirectoryDto::from)
                .toList();
    }
}
