package ru.javakira.simbirshare.storage;

import lombok.RequiredArgsConstructor;
import org.springframework.core.io.UrlResource;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.core.io.Resource;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.server.ResponseStatusException;
import ru.javakira.simbirshare.account.Account;
import ru.javakira.simbirshare.account.AccountService;
import ru.javakira.simbirshare.directory.Directory;
import ru.javakira.simbirshare.directory.DirectoryService;
import ru.javakira.simbirshare.file.File;
import ru.javakira.simbirshare.file.FileService;

import java.io.IOException;
import java.net.MalformedURLException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardCopyOption;
import java.util.Collections;
import java.util.List;

@Service
@RequiredArgsConstructor
public class StorageServiceImpl implements StorageService {
    private final StorageProperties properties;
    private final FileService fileService;
    private final DirectoryService directoryService;
    private final AccountService accountService;

    @Override
    public StorageInfo info(long userId) {
        Account account = accountService.account(userId);

        long used;
        Path root = path(account.getRoot());
        try {
            used = Files.walk(root)
                    .filter(p -> p.toFile().isFile())
                    .mapToLong(p -> p.toFile().length())
                    .sum() / 1024;
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        long free = account.getDiskSize() - used;

        return new StorageInfo(used, free);
    }

    @Override
    public Resource download(long fileId) {
        File file = fileService.file(fileId);

        try {
            Path filePath = path(file);
            return new UrlResource(filePath.toUri());
        }
        catch (MalformedURLException ignored) {
            throw new ResponseStatusException(
                    HttpStatus.INTERNAL_SERVER_ERROR,
                    "something wrong"
            );
        }
    }

    @Override
    public void upload(long fileId, MultipartFile fileValue) {
        File file = fileService.file(fileId);
        Directory directory = directoryService.directory(file.getDirectoryId());
        if (fileValue.getSize() > info(directory.getOwnerId()).getFree())
            throw new ResponseStatusException(
                    HttpStatus.BAD_REQUEST, //todo тут бы что нибудь умнее
                    "Storage doesn't have enough space for upload"
            );

        try {
            Path filePath = path(file);
            Files.copy(fileValue.getInputStream(), filePath, StandardCopyOption.REPLACE_EXISTING);
            file.setSize(Files.size(filePath));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private Path path(File file) {
        Directory directory = directoryService.directory(file.getDirectoryId());
        return path(directory).resolve(file.getName());
    }

    private Path path(Directory directory) {
        Path path = root();
        List<Directory> directories = directoryService.getDirectories(directory);
        Collections.reverse(directories);
        for(Directory directory1 : directories) {
            path = path.resolve(directory1.getName());
        }

        if (!Files.exists(path)) {
            try {
                Files.createDirectories(path);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
        return path;
    }

    private Path root() {
        return Path.of(properties.getStoragePath());
    }
}
